package edu.iastate.ece.afibdet;

import android.content.BroadcastReceiver;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.database.Cursor;
import android.net.Uri;
import android.os.Handler;
import android.os.HandlerThread;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;
import android.widget.ToggleButton;

import com.github.mikephil.charting.charts.LineChart;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;

public class HeartRate extends AppCompatActivity {

    private static final String SCROLL_TO_END = "SCROLL_TO_END";
    private static final String SCROLL_X_POS = "SCROLL_X_POS";

    HeartRateGraph heartRateGraph;
    BroadcastReceiver detectedReciever;
    HandlerThread detectedThread;
    Handler detectedHandler;
    TextView detectedLog;

    // only for demo
    long lastTime=0;

    //
    Runnable runnable;
    Handler handler;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_heart_rate);
        LineChart heartChart = (LineChart) findViewById(R.id.heart_rate_graph);
        heartRateGraph = new HeartRateGraph(heartChart, this);


        detectedLog = (TextView) findViewById(R.id.detected_log);

        if(savedInstanceState != null){
            boolean scroll = savedInstanceState.getBoolean(SCROLL_TO_END, true);
            ((ToggleButton) findViewById(R.id.scroll_to_end)).setChecked(scroll);
            heartRateGraph.setScrollToEnd(scroll);
            heartRateGraph.setX(savedInstanceState.getInt(SCROLL_X_POS,0));
        } else {
            ((ToggleButton) findViewById(R.id.scroll_to_end)).setChecked(true);
            heartRateGraph.setScrollToEnd(true);
        }

        detectedThread = new HandlerThread("NewDetectedData",
                android.os.Process.THREAD_PRIORITY_BACKGROUND);
        detectedThread.start();
        detectedHandler = new Handler(detectedThread.getLooper());

        detectedReciever = new BroadcastReceiver() {
            Context context;

            {
                context = getApplicationContext();
            }

            @Override
            public void onReceive(Context context, Intent intent) {
                // booleans in sqlite are integers
                String[] projection = {HeartRateDatabaseProvider.TIME};
                String query = HeartRateDatabaseProvider.DETECTED + "!=?";
                String[] queryArgs = {"0"};
                String ordering = HeartRateDatabaseProvider.TIME + " DESC";
                Cursor cur = context.getContentResolver().query(HeartRateDatabaseProvider.CONTENT_URI,projection,query,queryArgs,ordering);
                if(cur != null && cur.moveToLast()) {
                    StringBuilder sb = new StringBuilder();
                    SimpleDateFormat df = new SimpleDateFormat("MM-dd HH:mm:ss.SSS");
                    int time_idx = cur.getColumnIndex(HeartRateDatabaseProvider.TIME);
                    long lastTime = Long.MAX_VALUE;
                    while(!cur.isBeforeFirst()){
                        long curTime = cur.getLong(time_idx);
                        if(lastTime - curTime > 12){
                            sb.append(String.format(context.getString(R.string.detected_log_line),df.format(new Date(curTime))));
                        }
                        lastTime = curTime;
                    }
                    detectedLog.setText(sb.toString());
                    cur.close();
                }
            }
        };
        registerReceiver(detectedReciever,new IntentFilter("edu.iastate.ece.afibdet.NEW_DETECTED"),null,detectedHandler);

        /*
        // TODO remove when getting values from bt
        lastTime = 0;
        handler = new Handler();
        runnable = new Runnable() {
            Random rand = new Random();
            long lastTime = System.currentTimeMillis();
            @Override
            public void run() {
                Intent intent = new Intent("edu.iastate.ece.afibdet.NEW_DATA");
                intent.putExtra("time", lastTime);
                ContentValues contentValues;
                contentValues = new ContentValues();
                contentValues.put(HeartRateDatabaseProvider.TIME, lastTime);
                contentValues.put(HeartRateDatabaseProvider.VALUE, rand.nextInt(4096));
                //contentValues.put(HeartRateDatabaseProvider.OBSERVED,true);
                Uri u = getApplicationContext().getContentResolver().insert(HeartRateDatabaseProvider.CONTENT_URI, contentValues);
                lastTime += 6;
                contentValues = new ContentValues();
                contentValues.put(HeartRateDatabaseProvider.TIME,lastTime);
                contentValues.put(HeartRateDatabaseProvider.VALUE, rand.nextInt(4096));
                //contentValues.put(HeartRateDatabaseProvider.OBSERVED,true);
                getApplicationContext().getContentResolver().insert(HeartRateDatabaseProvider.CONTENT_URI, contentValues);
                lastTime += 6;
                contentValues = new ContentValues();
                contentValues.put(HeartRateDatabaseProvider.TIME,lastTime);
                contentValues.put(HeartRateDatabaseProvider.VALUE, rand.nextInt(4096));
                //contentValues.put(HeartRateDatabaseProvider.OBSERVED,true);
                getApplicationContext().getContentResolver().insert(HeartRateDatabaseProvider.CONTENT_URI, contentValues);
                lastTime += 6;
                contentValues = new ContentValues();
                contentValues.put(HeartRateDatabaseProvider.TIME,lastTime);
                contentValues.put(HeartRateDatabaseProvider.VALUE, rand.nextInt(4096));
                //contentValues.put(HeartRateDatabaseProvider.OBSERVED,true);
                getApplicationContext().getContentResolver().insert(HeartRateDatabaseProvider.CONTENT_URI, contentValues);
                lastTime += 6;
                contentValues = new ContentValues();
                contentValues.put(HeartRateDatabaseProvider.TIME,lastTime);
                contentValues.put(HeartRateDatabaseProvider.VALUE, rand.nextInt(4096));
                //contentValues.put(HeartRateDatabaseProvider.OBSERVED,true);
                getApplicationContext().getContentResolver().insert(HeartRateDatabaseProvider.CONTENT_URI, contentValues);
                lastTime += 6;
                contentValues = new ContentValues();
                contentValues.put(HeartRateDatabaseProvider.TIME,lastTime);
                contentValues.put(HeartRateDatabaseProvider.VALUE, rand.nextInt(4096));
                //contentValues.put(HeartRateDatabaseProvider.OBSERVED,true);
                getApplicationContext().getContentResolver().insert(HeartRateDatabaseProvider.CONTENT_URI, contentValues);
                lastTime += 6;
                sendBroadcast(intent);
                handler.postDelayed(this, 30);
            }
        };
        */
    }

    protected void onSaveInstanceState(Bundle outInstance){
        outInstance.putBoolean(SCROLL_TO_END,heartRateGraph.getScrollToEnd());
        outInstance.putInt(SCROLL_X_POS, heartRateGraph.getX());
    }

    @Override
    protected void onStart(){
        //runnable.run();
        super.onStart();
    }

    @Override
    protected void onStop(){
        //handler.removeCallbacks(runnable);

        detectedThread.quit();
        super.onStop();
    }

    @Override
    protected void onDestroy(){
        unregisterReceiver(detectedReciever);
        heartRateGraph.close();
        super.onDestroy();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_heart_rate, menu);

        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        /*
        if (id == R.id.action_settings) {
            return true;
        }
        */

        return super.onOptionsItemSelected(item);
    }

    public void toggleScrollToLatest(View view){
        ToggleButton toggleButton = (ToggleButton) view;
        heartRateGraph.setScrollToEnd(toggleButton.isChecked());
    }

    public void setupMonitor(View view){
        Intent intent = new Intent(this, SetupMonitor.class);
        startActivity(intent);
    }
}
