package edu.iastate.ece.afibdet;

import android.content.ContentProvider;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.database.sqlite.SQLiteOpenHelper;
import android.net.Uri;
import android.support.annotation.Nullable;

/**
 * Created by seth on 4/21/2016.
 */
public class HeartRateDatabaseProvider extends ContentProvider {

    static final String PROVIDER_NAME = "edu.iastate.ece.afibdet.provider.Heart";
    static final String URL = "content://" + PROVIDER_NAME + "/value";
    static final Uri CONTENT_URI = Uri.parse(URL);

    static final int HEART_VALUE = 1;
    static final int HEART_VALUE_LIMIT = 2;

    static final UriMatcher uriMatcher;
    static{
        uriMatcher = new UriMatcher(UriMatcher.NO_MATCH);
        uriMatcher.addURI(PROVIDER_NAME, "value", HEART_VALUE);
    }

    public static final String DATABASE_NAME = "heartrate.db";
    public static final String TABLE_NAME = "heartrate_table";
    public static final int HEART_RATE_TABLE_VER = 1;
    public static final String ID = "ID";
    public static final String TIME = "TIME";
    public static final String VALUE = "VALUE";
    public static final String OBSERVED = "OBSERVED";
    public static final String DETECTED = "DETECTED";

    /**
     * Helper class that actually creates and manages
     * the provider's underlying data repository.
     */
    private static class DatabaseHelper extends SQLiteOpenHelper {

        private static final String CREATE_TABLE = "CREATE TABLE " + TABLE_NAME + " ("
                + ID + " INTEGER PRIMARY KEY AUTOINCREMENT, "
                + TIME + " LONG, "
                + VALUE + " LONG, "
                + OBSERVED + " INTEGER, "
                + DETECTED +  " INTEGER)";

        private static final String DROP_TABLE = "DROP TABLE IF EXISTS " + TABLE_NAME;

        DatabaseHelper(Context context){
            super(context, DATABASE_NAME, null, HEART_RATE_TABLE_VER);
        }

        @Override
        public void onCreate(SQLiteDatabase db)
        {
            db.execSQL(CREATE_TABLE);
        }

        @Override
        public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
            db.execSQL(DROP_TABLE);
            onCreate(db);
        }
    }

    DatabaseHelper databaseHelper;
    SQLiteDatabase db;
    @Override
    public boolean onCreate() {
        databaseHelper = new DatabaseHelper(getContext());
        return true;
    }

    @Nullable
    @Override
    public Cursor query(Uri uri, String[] projection, String selection, String[] selectionArgs, String sortOrder) {
        switch (uriMatcher.match(uri)) {
            case HEART_VALUE:
                db = databaseHelper.getReadableDatabase();
                String limit = uri.getQueryParameter("limit");
                return db.query(TABLE_NAME, projection, selection, selectionArgs, null, null, sortOrder, limit);
            default:
                return null;
        }
    }

    @Nullable
    @Override
    public String getType(Uri uri) {
        switch (uriMatcher.match(uri)){
            case HEART_VALUE:
                return "vnd.android.cursor.dir/vnd.edu.iastate.edu.afibdet.Heart";
            default:
                throw new IllegalArgumentException("unsupported Uri: " + uri);
        }
    }

    @Nullable
    @Override
    public Uri insert(Uri uri, ContentValues values) {
        switch (uriMatcher.match(uri)){
            case HEART_VALUE:
                db = databaseHelper.getWritableDatabase();
                long id = db.insert(TABLE_NAME,null,values);
                if(id >= 0){
                    Uri ret = ContentUris.withAppendedId(CONTENT_URI,id);
                    return ret;
                }
                throw new SQLiteException("could not insert row URI: ["+ uri + "] error: " + id);
            default:
                throw new IllegalArgumentException("Unknown URI: " + uri);
        }
    }

    @Override
    public int delete(Uri uri, String selection, String[] selectionArgs) {
        switch(uriMatcher.match(uri)){
            case HEART_VALUE:
                db = databaseHelper.getWritableDatabase();
                int ret = db.delete(TABLE_NAME,selection,selectionArgs);
                return ret;
            default:
                throw new IllegalArgumentException("Unknown URI: " + uri);
        }
    }

    @Override
    public int update(Uri uri, ContentValues values, String selection, String[] selectionArgs) {
        switch(uriMatcher.match(uri)){
            case HEART_VALUE:
                db = databaseHelper.getWritableDatabase();
                int ret = db.update(TABLE_NAME,values,selection,selectionArgs);
                return ret;
            default:
                throw new IllegalArgumentException("Unknown URI: " + uri);
        }
    }
}