package edu.iastate.ece.afibdet;

import android.content.BroadcastReceiver;
import android.content.ContentProviderClient;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.database.Cursor;
import android.graphics.Color;
import android.graphics.Typeface;
import android.net.Uri;
import android.os.*;
import android.util.Log;

import com.github.mikephil.charting.charts.LineChart;
import com.github.mikephil.charting.components.XAxis;
import com.github.mikephil.charting.components.YAxis;
import com.github.mikephil.charting.data.Entry;
import com.github.mikephil.charting.data.LineData;
import com.github.mikephil.charting.data.LineDataSet;
import com.github.mikephil.charting.interfaces.datasets.ILineDataSet;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * Created by seth on 4/12/2016.
 */
public class HeartRateGraph {
    private static final Uri DB_URI = HeartRateDatabaseProvider.CONTENT_URI;

    private LineChart heartChart;
    private boolean scrollToEnd;
    private Context context;
    private long lastTime;
    private SimpleDateFormat df;

    private int numToShow = 200;

    HandlerThread observedThread;
    Handler observedHandler;

    private BroadcastReceiver heartRateDataReciever;

    HeartRateGraph(LineChart heartChart, Context _context) {
        this.scrollToEnd = true;
        this.heartChart = heartChart;
        this.context = _context;
        observedThread = new HandlerThread("NewObservedData",
                android.os.Process.THREAD_PRIORITY_URGENT_DISPLAY);
        observedThread.start();
        Looper looper = observedThread.getLooper();
        observedHandler = new Handler(looper);
        init();
    }

    private Cursor getObserved(int limit){
        String[] proj = {HeartRateDatabaseProvider.TIME,HeartRateDatabaseProvider.VALUE,HeartRateDatabaseProvider.DETECTED};
        String query = HeartRateDatabaseProvider.OBSERVED + "!=?";
        String[] queryArgs = {"0"};
        String ordering = HeartRateDatabaseProvider.TIME + " ASC";
        Uri observedUri;
        if(limit != Integer.MAX_VALUE){
            observedUri = DB_URI.buildUpon().appendQueryParameter("limit",Integer.toString(limit)).build();
        } else {
            observedUri = DB_URI;
        }

        return context.getContentResolver().query(observedUri,proj,query,queryArgs,ordering);
    }

    private Cursor getObservedAfter(long time){
        String[] proj = {HeartRateDatabaseProvider.TIME,HeartRateDatabaseProvider.VALUE,HeartRateDatabaseProvider.DETECTED};
        String query = HeartRateDatabaseProvider.OBSERVED + "!=? And " +
                HeartRateDatabaseProvider.TIME + ">?";
        String[] queryArgs = {"0",Long.toString(time)};
        String ordering = HeartRateDatabaseProvider.TIME + " ASC";

        return context.getContentResolver().query(DB_URI,proj,query,queryArgs,ordering);
    }

    private void init(){
        df = new SimpleDateFormat("dd HH:mm:ss.SSSS");
        heartChart.setDescription("");
        //TODO: stings file
        heartChart.setNoDataTextDescription("A monitor needs to be paired");

        heartChart.setTouchEnabled(true);
        heartChart.setScaleEnabled(true);
        heartChart.setDragEnabled(true);
        heartChart.setDrawGridBackground(false);
        heartChart.setPinchZoom(false);
        //TODO: colors file
        heartChart.setBackgroundColor(Color.WHITE);
        //TODO: strings file
        LineDataSet set = new LineDataSet(null, "Heart Beat");
        set.setAxisDependency(YAxis.AxisDependency.LEFT);
        set.setColor(Color.RED);
        set.setDrawCircles(false);
        set.setLineWidth(2f);
        set.setFillAlpha(65);
        set.setFillColor(Color.RED);
        set.setHighLightColor(Color.rgb(244, 117, 117));
        set.setHighlightEnabled(false);
        set.setValueTextColor(Color.WHITE);
        set.setValueTextSize(9f);
        set.setDrawValues(false);

        LineData data = new LineData();
        data.addDataSet(set);

        Cursor cur = getObserved(numToShow);
        if(cur.moveToLast() && !cur.isAfterLast()){
            lastTime = cur.getInt(cur.getColumnIndex(HeartRateDatabaseProvider.TIME));
        } else {
            lastTime = 0;
        }
        cur.moveToFirst();
        int time_idx = cur.getColumnIndex(HeartRateDatabaseProvider.TIME);
        int val_idx = cur.getColumnIndex(HeartRateDatabaseProvider.VALUE);
        while(!cur.isAfterLast()){
            data.addXValue(df.format(new Date(cur.getLong(time_idx))));
            set.addEntry(new Entry((float) cur.getInt(val_idx), set.getEntryCount()));
            cur.moveToNext();
        }
        cur.close();
        heartChart.notifyDataSetChanged();
        heartChart.setVisibleXRangeMaximum(numToShow);
        heartChart.moveViewToX(data.getXValCount() - (numToShow+1));

        data.setValueTextColor(Color.WHITE);

        heartChart.setData(data);
        //hide legend
        heartChart.getLegend().setEnabled(false);

        //Xaxis
        XAxis xAxis = heartChart.getXAxis();
        //xAxis.setValueFormatter();
        xAxis.setTypeface(Typeface.DEFAULT);
        xAxis.setDrawGridLines(false);
        xAxis.setAvoidFirstLastClipping(true);
        xAxis.setSpaceBetweenLabels(5);
        xAxis.setPosition(XAxis.XAxisPosition.BOTTOM);
        xAxis.setEnabled(true);

        final YAxis leftAxis = heartChart.getAxisLeft();
        leftAxis.setTypeface(Typeface.DEFAULT);
        leftAxis.setAxisMinValue(-5f);
        leftAxis.setAxisMaxValue(4100f);
        leftAxis.setDrawLabels(false);
        leftAxis.setDrawGridLines(true);

        heartChart.getAxisRight().setEnabled(false);

        heartRateDataReciever = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                Log.i("broadcastreciev","onRecieve");
                    LineData data = heartChart.getLineData();
                    ILineDataSet set = data.getDataSetByIndex(0);

                    Cursor cur = getObservedAfter(lastTime);
                    cur.moveToFirst();
                    while (!cur.isAfterLast()) {
                        long time = cur.getLong(cur.getColumnIndex(HeartRateDatabaseProvider.TIME));
                        int val = cur.getInt(cur.getColumnIndex(HeartRateDatabaseProvider.VALUE));
                        data.addXValue(df.format(new Date(time)));
                        set.addEntry(new Entry((float) val, set.getEntryCount()));
                        lastTime = time;
                        cur.moveToNext();
                    }
                    cur.close();
                    heartChart.notifyDataSetChanged();
                    heartChart.setVisibleXRangeMaximum(numToShow);
                    heartChart.moveViewToX(data.getXValCount() - (numToShow + 1));
            }
        };

        context.registerReceiver(heartRateDataReciever, new IntentFilter("edu.iastate.ece.afibdet.NEW_OBSERVED"),null,observedHandler);
    }

    private void fillGraph(int records){
            LineData data = heartChart.getLineData();
            ILineDataSet set = data.getDataSetByIndex(0);
            set.clear();
            for (int i = data.getXValCount() - 1; i >= 0; i--) {
                data.removeXValue(i);
            }
            // as many rows as we can fit into the graph

            Cursor cur = getObserved(records);
            if(cur.moveToLast() && !cur.isAfterLast()){
                lastTime = cur.getInt(cur.getColumnIndex(HeartRateDatabaseProvider.TIME));
            } else {
                lastTime = 0;
            }
            cur.moveToFirst();
            int time_idx = cur.getColumnIndex(HeartRateDatabaseProvider.TIME);
            int val_idx = cur.getColumnIndex(HeartRateDatabaseProvider.VALUE);
            while(!cur.isAfterLast()){
                data.addXValue(df.format(new Date(cur.getLong(time_idx))));
                set.addEntry(new Entry((float) cur.getInt(val_idx), set.getEntryCount()));
                cur.moveToNext();
            }
            cur.close();
            heartChart.notifyDataSetChanged();
            heartChart.setVisibleXRangeMaximum(numToShow);
            heartChart.moveViewToX(data.getXValCount() - (numToShow+1));

    }

    public void close(){
        if(scrollToEnd) {
            context.unregisterReceiver(heartRateDataReciever);
        }
        observedThread.quit();
    }

    public boolean getScrollToEnd() {
        return scrollToEnd;
    }

    public void setScrollToEnd(boolean scrollToEnd) {
        if(this.scrollToEnd && !scrollToEnd) {
            // switching to static data view
            this.scrollToEnd = false;
            context.unregisterReceiver(heartRateDataReciever);
            fillGraph(Integer.MAX_VALUE);
        } else if(!this.scrollToEnd && scrollToEnd){
            // switching to dynamic new data view
            this.scrollToEnd = true;
            fillGraph(numToShow);
            context.registerReceiver(heartRateDataReciever, new IntentFilter("edu.iastate.ece.afibdet.NEW_OBSERVED"),null,observedHandler);
        }
    }

    public int getX() {
        return this.heartChart.getScrollX();

    }

    public void setX(int x){
        this.heartChart.setScrollX(x);
    }
}
