#include <msp430.h>
#include "SPP.h"
#include "SS1BTPS.h"             /* Main SS1 Bluetooth Stack Header.          */
#include "string.h"

static volatile unsigned int A0results;
static volatile unsigned int A1results;
static volatile unsigned int A2results;
static volatile unsigned int A3results;
static volatile unsigned int A4results;
static volatile unsigned int A5results;
static volatile unsigned int A6results;
static volatile unsigned int A7results;
static volatile unsigned int A8results;
static volatile unsigned int A9results;
static volatile unsigned int A10results;
static volatile unsigned int A11results;
static volatile unsigned int A12results;
static volatile unsigned int A13results;
static volatile unsigned int A14results;
static volatile unsigned int A15results;

/* TODO find good size for the circular buffer */
#define ADC_CIRC_BUFF_SIZE 1024
struct {
	char queued;
	int toRead;
	int toWrite;
	char values[ADC_CIRC_BUFF_SIZE];
} circ_buff;

static char fresh = 0;

char adc_buff[1024];

/* do not ever write more than ADC_CIRC_BUFF_SIZE bytes with this function */
void write_circ(int len, char *buff) {
	/* we do no overwrite protection here, may want to advance the first pointer?? */
	if (circ_buff.toWrite >= ADC_CIRC_BUFF_SIZE) {
		// we are at the end of the buffer
		circ_buff.toWrite = 0;
	}

	if (circ_buff.toWrite + len <= ADC_CIRC_BUFF_SIZE) {
		// outer buffer is limiting
		memcpy(&(circ_buff.values[circ_buff.toWrite]), buff, len);
		circ_buff.toWrite += len;
		// edge gaurd
		if (circ_buff.toWrite >= ADC_CIRC_BUFF_SIZE) {
			// we are at the end of the buffer
			circ_buff.toWrite = 0;
		}
	} else {
		int toWrite = ADC_CIRC_BUFF_SIZE - circ_buff.toWrite; // len first write
		int toWrite2 = len - toWrite; // len second write
		// write the first section up to the end of the buffer
		memcpy(&(circ_buff.values[circ_buff.toWrite]), buff, toWrite);
		// write the second section
		memcpy(&(circ_buff.values[0]), &(buff[toWrite]), toWrite2);
		// update once
		circ_buff.toWrite = toWrite2;
	}
}

int read_circ(int len, char *buff) {
	if (circ_buff.toRead >= ADC_CIRC_BUFF_SIZE) {
		circ_buff.toRead = 0;
	}
	if (circ_buff.toRead <= circ_buff.toWrite) {
		// we have not wrapped
		int toMove = circ_buff.toWrite - circ_buff.toRead;
		if (toMove <= len) {
			// circ buff is limiting
			memcpy(buff, &(circ_buff.values[circ_buff.toRead]), toMove);
			circ_buff.toRead += toMove;
			return toMove;
		} else {
			// out buff is limiting
			memcpy(buff, &(circ_buff.values[circ_buff.toRead]), len);
			circ_buff.toRead += len;
			return len;
		}
	} else {
		// we have wrapped
		int toMove = ADC_CIRC_BUFF_SIZE - circ_buff.toRead;
		if (toMove <= len) {
			// circ buff is limiting
			memcpy(buff, &(circ_buff.values[circ_buff.toRead]), toMove);
			// we have hit the end of the buffer
			circ_buff.toRead = 0;
			// TODO: second copy??
			return toMove;
		} else {
			// out buff is limiting
			memcpy(buff, &(circ_buff.values[circ_buff.toRead]), len);
			circ_buff.toRead += len;
			return len;
		}
	}
}

void configureADC(void) {
	circ_buff.queued = 0;
	circ_buff.toRead = 0;
	circ_buff.toWrite = 0;
	P6SEL = 0x80;
	/* sample and hold at 987 to result in conversion time of 1000 cycles */
	/* Turn on ADC 12 with extend sampling time to avoid overflow of results */
	ADC12CTL0 = 0x0D00 + ADC12ON + ADC12MSC + ADC12SHT0_8;
	/* sampling timer, repeated sequence sampling, clock divider of 5 (ADC12DIV_4 is 5) */
	ADC12CTL1 = ADC12SHP + ADC12CONSEQ_3 + ADC12DIV_7;
	/* 12 bit resolution , temperature sensor off, (refout is set to zero)*/
	ADC12CTL2 = ADC12RES_2 + ADC12TCOFF + ADC12PDIV;
	/* all comming from ch7 with +Vref */
	ADC12MCTL0 = ADC12INCH_7 + ADC12SREF1;
	ADC12MCTL1 = ADC12INCH_7 + ADC12SREF1;
	ADC12MCTL2 = ADC12INCH_7 + ADC12SREF1;
	ADC12MCTL3 = ADC12INCH_7 + ADC12SREF1;
	ADC12MCTL4 = ADC12INCH_7 + ADC12SREF1;
	ADC12MCTL5 = ADC12INCH_7 + ADC12SREF1;
	ADC12MCTL6 = ADC12INCH_7 + ADC12SREF1;
	ADC12MCTL7 = ADC12INCH_7 + ADC12SREF1;
	ADC12MCTL8 = ADC12INCH_7 + ADC12SREF1;
	ADC12MCTL9 = ADC12INCH_7 + ADC12SREF1;
	ADC12MCTL10 = ADC12INCH_7 + ADC12SREF1;
	ADC12MCTL11 = ADC12INCH_7 + ADC12SREF1;
	ADC12MCTL12 = ADC12INCH_7 + ADC12SREF1;
	ADC12MCTL13 = ADC12INCH_7 + ADC12SREF1;
	ADC12MCTL14 = ADC12INCH_7 + ADC12SREF1;
	ADC12MCTL15 = ADC12INCH_7 + ADC12SREF1 + ADC12EOS;
	/* enable interrupt for memory 15 */
	ADC12IE = ADC12IFG15;
}

void stopADC(void) {
	/* Enable conversions */
	ADC12CTL0 &= ~ADC12ENC;
	/* start conversion */
	ADC12CTL0 &= ~ADC12SC;
}

void startADC(void) {
	/* Enable conversions */
	ADC12CTL0 |= ADC12ENC;
	/* start conversion */
	ADC12CTL0 |= ADC12SC;
}

void printADC(void *UserParameter) {
	char data[32];
	int len = read_circ(32, data);
	static int zeros = 0;
	if (len != 0) {
		serialSendData(len, (Byte_t *) data);
	} else {
		zeros++;
	}
}

void sendHeartBeat(void *UserParameter){
	char data[2];
	data[0] = 0x80 | '~';
	data[1] = 'S';
	serialSendData(2, (Byte_t *) data);
}

/* Sequence of bytes
 MSB                          8   7                          LSB
 +---+---+---+---+---+---+---+---|---+---+---+---+---+---+---+---+
 |con|    seq    |     ADC_h     |              ADC_l            |
 +---+---+---+---+---+---+---+---|---+---+---+---+---+---+---+---+

 Example
 MSB                          8   7                          LSB
 +---+---+---+---+---+---+---+---|---+---+---+---+---+---+---+---+
 | 0 | 0   0   0 | ADC_read_0_h  |          ADC_read_0_l         |
 +---+---+---+---+---+---+---+---|---+---+---+---+---+---+---+---+
 | 0 | 0   0   1 | ADC_read_1_h  |          ADC_read_1_l         |
 +---+---+---+---+---+---+---+---|---+---+---+---+---+---+---+---+
 | 0 | 0   1   0 | ADC_read_2_h  |          ADC_read_2_l         |
 +---+---+---+---+---+---+---+---|---+---+---+---+---+---+---+---+
 | 0 | 0   1   1 | ADC_read_3_h  |          ADC_read_3_l         |
 +---+---+---+---+---+---+---+---|---+---+---+---+---+---+---+---+
 | 0 | 1   0   0 | ADC_read_4_h  |          ADC_read_4_l         |
 +---+---+---+---+---+---+---+---|---+---+---+---+---+---+---+---+
 | 0 | 1   0   1 | ADC_read_5_h  |          ADC_read_5_l         |
 +---+---+---+---+---+---+---+---|---+---+---+---+---+---+---+---+
 | 0 | 1   1   0 | ADC_read_6_h  |          ADC_read_6_l         |
 +---+---+---+---+---+---+---+---|---+---+---+---+---+---+---+---+
 | 0 | 1   1   1 | ADC_read_7_h  |          ADC_read_7_l         |
 +---+---+---+---+---+---+---+---|---+---+---+---+---+---+---+---+

 Resync message, should be sent every second `~S`
 +---+---+---+---+---+---+---+---|---+---+---+---+---+---+---+---+
 | 1   1   1   1   1   1   1   0   0   1   0   1   0   0   1   1 |
 +---+---+---+---+---+---+---+---|---+---+---+---+---+---+---+---+
 */

/* ADC12 interrupt vector */

#pragma vector=ADC12_VECTOR
__interrupt void ADC12ISR(void) {
	char reads[32];
	switch (ADC12IV) {
	case 0:
		break;                           // Vector  0:  No interrupt
	case ADC12IV_ADC12IFG15:
		reads[0] = 0x00 | ADC12MEM0_H;
		reads[1] = ADC12MEM0_L;

		reads[2] = 0x10 | ADC12MEM1_H;
		reads[3] = ADC12MEM1_L;

		reads[4] = 0x20 | ADC12MEM2_H;
		reads[5] = ADC12MEM2_L;

		reads[6] = 0x30 | ADC12MEM3_H;
		reads[7] = ADC12MEM3_L;

		reads[8] = 0x40 | ADC12MEM4_H;
		reads[9] = ADC12MEM4_L;

		reads[10] = 0x50 | ADC12MEM5_H;
		reads[11] = ADC12MEM5_L;

		reads[12] = 0x60 | ADC12MEM6_H;
		reads[13] = ADC12MEM6_L;

		reads[14] = 0x70 | ADC12MEM7_H;
		reads[15] = ADC12MEM7_L;

		reads[16] = 0x00 | ADC12MEM8_H;
		reads[17] = ADC12MEM8_L;

		reads[18] = 0x10 | ADC12MEM9_H;
		reads[19] = ADC12MEM9_L;

		reads[20] = 0x20 | ADC12MEM10_H;
		reads[21] = ADC12MEM10_L;

		reads[22] = 0x30 | ADC12MEM11_H;
		reads[23] = ADC12MEM11_L;

		reads[24] = 0x40 | ADC12MEM12_H;
		reads[25] = ADC12MEM12_L;

		reads[26] = 0x50 | ADC12MEM13_H;
		reads[27] = ADC12MEM13_L;

		reads[28] = 0x60 | ADC12MEM14_H;
		reads[29] = ADC12MEM14_L;

		reads[30] = 0x70 | ADC12MEM15_H;
		reads[31] = ADC12MEM15_L;

		// 16 ints is 32 bytes
		write_circ(32, (char *) reads);
		break;
	default:
		break;
	}
}

